/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "RemoteWebInspectorUI.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "RemoteWebInspectorUIMessages.h"
#include <wtf/text/WTFString.h>

namespace WebKit {

void RemoteWebInspectorUI::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteWebInspectorUI::Initialize::name()) {
        IPC::handleMessage<Messages::RemoteWebInspectorUI::Initialize>(decoder, this, &RemoteWebInspectorUI::initialize);
        return;
    }
    if (decoder.messageName() == Messages::RemoteWebInspectorUI::DidSave::name()) {
        IPC::handleMessage<Messages::RemoteWebInspectorUI::DidSave>(decoder, this, &RemoteWebInspectorUI::didSave);
        return;
    }
    if (decoder.messageName() == Messages::RemoteWebInspectorUI::DidAppend::name()) {
        IPC::handleMessage<Messages::RemoteWebInspectorUI::DidAppend>(decoder, this, &RemoteWebInspectorUI::didAppend);
        return;
    }
    if (decoder.messageName() == Messages::RemoteWebInspectorUI::SendMessageToFrontend::name()) {
        IPC::handleMessage<Messages::RemoteWebInspectorUI::SendMessageToFrontend>(decoder, this, &RemoteWebInspectorUI::sendMessageToFrontend);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit
